/*
  $Id: _gawk.c 47 2009-08-03 15:46:35Z carlo $

  _gawk: SWU gawk(1) front-end.

  Copyright (c) 2001,2008 Carlo Strozzi

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*

 The need for this program arises from the fact that if gawk(1) is
 passed options that it recognizes, it will intercept them instead of
 passing them to the AWK script through ARGV/ARGC, unless they are
 preceeded by either a non-gawk option or '--'. This makes it impossible
 to pass valid gawk(1) options to the AWK program if the latter is a
 self-contained executable script beginning with '#!/path/to/gawk -f'.
 The '-We' mawk(1) option, or something equivalent, is not available
 with gawk(1), and short of other tricks I made this wrapper to overcome
 the problem. Unfortunately this introduces one more exec(3) for each
 AWK program invocation, so at the moment mawk(1) is still preferable
 over gawk(1).

 Additionally, using this wrapper provides some extra security when
 gawk(1) is used a a Web CGI processor. One of gawk's features is
 that file scripts and inline scripts can be mixed on the command-line,
 so it may be possible to inject AWK code from the CGI interface.
 To prevent this, gawk's documentation recommends that option '-W exec'
 is used, but not all versions of gawk support it, so using this wrapper
 provides both security and portability.

*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/file.h>
#include <string.h>
#include "util.h"

#define MAXFNAME 256

int main(int argc, char *argv[]) {

    int i=1;

    Strlist *args;

    args = strlist();
    slappend(args, "gawk");
    slappend(args, "-f");

    if (argc < 2) {
       fprintf(stderr, "Usage: _gawk program-file [program-arguments]\n");
       exit(2);
    }

    slappend(args, argv[1]);
    slappend(args, "--");

    while (++i < argc) slappend(args, argv[i]);

    execvp("gawk", sl2vec(args));

    perror("gawk");
    exit(2);
}

/* EOF */
